#!/usr/bin/env node
'use strict';

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _purdy = require('purdy');

var _purdy2 = _interopRequireDefault(_purdy);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _load = require('./load');

var _load2 = _interopRequireDefault(_load);

var _ipc = require('./ipc');

var _ipc2 = _interopRequireDefault(_ipc);

var _web = require('./platform/web');

var _web2 = _interopRequireDefault(_web);

var _node = require('./platform/node');

var _node2 = _interopRequireDefault(_node);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var options = {
  stats: {
    hash: false,
    cached: false,
    cachedAssets: false,
    colors: true,
    modules: false,
    chunks: false
  }
};

var token = Math.random().toString(36).substr(2);
var argv = _yargs2.default.argv;

global.__IN_DEV_SERVER = true;

var file = argv._[0];
var config = function () {
  console.log('⏳  Loading ' + _path2.default.basename(file) + '...');
  return (0, _load2.default)(file);
}();

var isDirectory = function isDirectory(path) {
  return path.charAt(path.length - 1) === '/';
};

// https://github.com/webpack/webpack/blob
// /1b9e880f388f49bc88b52d5a6bbab5538d4c311e
// /lib/JsonpMainTemplate.runtime.js#L27
if (!config.output.publicPath) {
  config.output.publicPath = '/';
}if (!isDirectory(config.output.publicPath)) {
  config.output.publicPath += '/';
}

config.plugins.push(new _webpack2.default.DefinePlugin({
  __webpack_dev_token__: JSON.stringify(token), // eslint-disable-line
  'process.env.IPC_URL': JSON.stringify(process.env.IPC_URL)
}));

var compiler = (0, _webpack2.default)(config);

(0, _web2.default)(compiler);
(0, _node2.default)(compiler);

// TODO: Figure out a better way of doing this.
if (process.env.DUMP_WEBPACK) {
  console.log('=====> ' + _path2.default.basename(file));
  console.log(_purdy2.default.stringify(config, {
    plain: false,
    indent: 2
  }));
}

console.log('🔨  Compiling ' + _path2.default.basename(file) + '...');
compiler.watch({}, function (err, stats) {
  if (err) {
    console.error(err);
    process.exit(231);
    return;
  }
  var data = stats.toJson();
  data.file = file;
  data.token = token;
  data.outputPath = compiler.outputPath;
  _ipc2.default.emit('stats', data);

  console.log(stats.toString(options.stats));
});
//# sourceMappingURL=compiler.js.map