'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _child_process = require('child_process');

var _backo = require('backo');

var _backo2 = _interopRequireDefault(_backo);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('../util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (compiler) {
  if (compiler.options.target !== 'node') {
    return null;
  }

  var backoff = new _backo2.default({ min: 0, max: 1000 * 5 });
  var child = null;
  var rip = false;
  var stats = null;
  var spawnTimeout = null;

  var kill = function kill() {
    if (spawnTimeout) {
      clearTimeout(spawnTimeout);
    }
    if (child) {
      (0, _util.kill)(child);
    }
  };

  var _spawn = function _spawn() {
    var entries = stats.chunks.filter(function (chunk) {
      return chunk.entry;
    });
    var env = _extends({}, process.env, {
      // Make sure apps don't try to steal our port.
      PORT: 0
    });

    // Only support one entrypoint right now. Maybe support more later.
    if (entries.length !== 1) {
      throw new Error('Must only export 1 entrypoint!');
    }
    var target = _path2.default.join(compiler.outputPath, entries[0].files[0]);
    console.log('▶️  Launching ' + target + '...');
    child = (0, _child_process.fork)(target, [], { env: env });
    child.once('exit', function (code) {
      if (rip) {
        // events.emit('close');
      } else {
          if (code === 218) {
            console.log('🆗  Restart via HMR.');
          }
          spawn();
        }
    });
    child.once('error', function () {
      if (rip) {
        // events.emit('close');
      } else {
          spawn();
        }
    });
  };

  var spawn = function spawn() {
    if (spawnTimeout) {
      clearTimeout(spawnTimeout);
    }
    kill();
    spawnTimeout = setTimeout(function () {
      return _spawn();
    }, backoff.duration());
  };

  process.once('beforeExit', function () {
    rip = true;
    kill();
  });

  compiler.plugin('done', function (_stats) {
    stats = _stats.toJson();
    if (!child) {
      spawn();
    }
  });

  return {
    close: function close() {
      rip = true;
      kill();
    }
  };
};
//# sourceMappingURL=node.js.map