'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _util = require('./util');

var _ipc = require('../ipc');

var _ipc2 = _interopRequireDefault(_ipc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global __webpack_hash__, __webpack_dev_token__ */

var ok = function ok(stats) {
  return !stats.errors || stats.errors.length === 0;
};

exports.default = function (_ref) {
  var reload = _ref.reload;

  var lastHash = '';
  var lastStats = null;
  var upToDate = function upToDate() {
    return lastHash.indexOf(__webpack_hash__) >= 0;
  };

  // Monitor our own stats for changes. Other services may call `watch` for
  // things they are interested in.
  _ipc2.default.emit('watch-stats', __webpack_dev_token__);

  _ipc2.default.on('stats', function (stats) {
    // Ignore everything but the update we want. Other stats may end up
    // getting sent to this app for other reasons.
    if (stats.token !== __webpack_dev_token__) {
      // eslint-disable-line
      return;
    }

    lastStats = stats;

    if (!ok(stats)) {
      console.log('🔥  Errors in build – refusing to reload.');
      return;
    }

    lastHash = stats.hash;
    if (!upToDate()) {
      if (module.hot) {
        if (module.hot.status() === 'idle') {
          console.log('🔥  Checking for updates...');
          check();
        }
      } else {
        reload();
      }
    } else {
      console.log('🔥  Up-to-date!');
    }
  });

  var checkz = function checkz() {
    return new Promise(function (resolve, reject) {
      module.hot.check(function (err, updates) {
        return err ? reject(err) : resolve(updates);
      });
    });
  };

  var applyz = function applyz(opts) {
    return new Promise(function (resolve, reject) {
      module.hot.apply(opts, function (err, result) {
        return err ? reject(err) : resolve(result);
      });
    });
  };

  var check = function check() {
    checkz().then(function (updatedModules) {
      if (!updatedModules) {
        console.warn('🔥  Cannot find update. Need to do a full reload!');
        reload();
        return;
      }
      applyz({ ignoreUnaccepted: true }).then(function (renewedModules) {
        if (!upToDate()) {
          check();
        }
        var unacceptedModules = (0, _util.unaccepted)(renewedModules, updatedModules);
        if (unacceptedModules.length > 0) {
          console.log('🔥  Unable to hot reload modules:');
          console.log(unacceptedModules.map(function (id) {
            if (lastStats && lastStats.modules && lastStats.modules[id]) {
              return ' 🚫  ' + lastStats.modules[id].name;
            }
            return id;
          }).join('\n'));
          reload();
        } else {
          console.log('🔥  Update applied.');
        }

        if (upToDate()) {
          console.log('🔥  App is up to date.');
        }
      }).catch(function (err) {
        console.error(err, module, '🔥  Update application failed.');
        reload();
      });
    }).catch(function (err) {
      console.error(err, module, '🔥  Update check failed.');
    });
  };
};
//# sourceMappingURL=common.js.map